<?php

namespace Drupal\simple_activecampaign\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\simple_activecampaign\ActiveCampaignApi;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Simple ActiveCampaign settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * ActiveCampaign API service.
   *
   * @var \Drupal\simple_activecampaign\ActiveCampaignApi
   */
  protected ActiveCampaignApi $activeCampaignApi;

  /**
   * Form constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\simple_activecampaign\ActiveCampaignApi $active_campaign_api
   *   The ActiveCampaign API service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ActiveCampaignApi $active_campaign_api) {
    parent::__construct($config_factory);
    $this->activeCampaignApi = $active_campaign_api;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('simple_activecampaign.api'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simple_activecampaign_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['simple_activecampaign.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('simple_activecampaign.settings');

    // Fetch contact lists from ActiveCampaign.
    $activecampaign_contact_lists = $this->activeCampaignApi->getContactLists();
    if (!is_array($activecampaign_contact_lists)) {
      $this->messenger()->addWarning($this->t('Unable to fetch contact lists from ActiveCampaign. Please contact website support.'));
      return [];
    }

    // Merge configuration with fetched contact lists.
    $contact_lists = [];
    $contact_lists_config = $config->get('contact_lists');
    foreach ($activecampaign_contact_lists as $contact_list) {
      $contact_lists[$contact_list->id] = [
        'name' => $contact_list->name,
        'label' => $contact_lists_config[$contact_list->id]['label'] ?? $contact_list->name,
        'weight' => $contact_lists_config[$contact_list->id]['weight'] ?? 0,
        'hide' => $contact_lists_config[$contact_list->id]['hide'] ?? 0,
      ];
    }

    $form['contact_lists'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Contact lists'),
      '#group' => 'settings',
    ];
    $form['contact_lists']['default_contact_list'] = [
      '#type' => 'select',
      '#title' => $this->t('Default contact list'),
      '#empty_option' => $this->t('- None -'),
      '#default_value' => $config->get('default_contact_list'),
      '#options' => array_map(function ($contact_list) {
        return $contact_list['name'];
      }, $contact_lists),
      '#description' => $this->t('All the subscribers will be added to this list by default.'),
    ];

    $form['contact_lists']['contact_lists'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Name'),
        $this->t('Description'),
        $this->t('Hide'),
        $this->t('Weight'),
      ],
      '#empty' => $this->t('No ActiveCampaign lists were found. Please check ActiveCampaign admin panel!'),
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'table-sort-weight',
        ],
      ],
    ];

    // Sort contact lists by weight.
    uasort($contact_lists, function ($a, $b) {
      return $a['weight'] - $b['weight'];
    });
    foreach ($contact_lists as $contact_list_id => $contact_list) {
      $form['contact_lists']['contact_lists'][$contact_list_id] = [
        '#attributes' => ['class' => 'draggable'],
        '#weight' => $contact_list['weight'],
        'name' => [
          '#markup' => $contact_list['name'],
        ],
        'label' => [
          '#type' => 'textfield',
          '#default_value' => $contact_list['label'],
          '#required' => TRUE,
        ],
        'hide' => [
          '#type' => 'checkbox',
          '#title' => $this->t('Hide @title', ['@title' => $contact_list['name']]),
          '#default_value' => $contact_list['hide'],
          '#title_display' => 'invisible',
          // If the list is selected as default. It will be hidden.
          '#states' => [
            'checked' => [
              ':input[name="default_contact_list"]' => ['value' => $contact_list_id],
            ],
            'disabled' => [
              ':input[name="default_contact_list"]' => ['value' => $contact_list_id],
            ],
          ],
        ],
        'weight' => [
          '#type' => 'weight',
          '#title' => $this->t('Weight for @title', ['@title' => $contact_list['name']]),
          '#title_display' => 'invisible',
          '#default_value' => $contact_list['weight'],
          '#attributes' => ['class' => ['table-sort-weight']],
        ],
      ];
    }

    $form['form_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Form settings'),
      '#group' => 'settings',
    ];
    $form['form_settings']['email_field_label'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $config->get('email_field_label') ?? $this->t('Email'),
      '#title' => $this->t('Email field label'),
    ];
    $form['form_settings']['contact_list_field_label'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $config->get('contact_list_field_label') ?? $this->t('Select a list'),
      '#title' => $this->t('Contact list field label'),
    ];
    $agreement_checkbox_label = $config->get('agreement_checkbox_label');
    $form['form_settings']['agreement_checkbox_label'] = [
      '#type' => 'text_format',
      '#required' => TRUE,
      '#rows' => 1,
      '#format' => $agreement_checkbox_label['format'] ?? '',
      '#default_value' => $agreement_checkbox_label['value'] ?? $this->t("I've read and agree to your privacy policy."),
      '#title' => $this->t('Agreement checkbox label'),
    ];
    $form['form_settings']['submit_button_label'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $config->get('submit_button_label') ?? $this->t('Subscribe'),
      '#title' => $this->t('Subscribe button label'),
    ];
    $form['form_settings']['successful_message'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $config->get('successful_message') ?? $this->t('Thank you for subscribing to our newsletter.'),
      '#title' => $this->t('Successful subscription message'),
    ];
    $form['form_settings']['failed_message'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $config->get('failed_message') ?? $this->t('Unable to finish subscription. Please contact website support.'),
      '#title' => $this->t('Failed subscription message'),
    ];

    // TODO:
    // - Allow hiding contact lists field.
    // - Allow hiding agreement field and editing its label.
    // - Allow showing/hiding firstName field and editing its label.
    // - Allow showing/hiding lastName field and editing its label.
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('simple_activecampaign.settings')
      ->set('default_contact_list', $form_state->getValue('default_contact_list'))
      ->set('contact_lists', $form_state->getValue('contact_lists'))
      ->set('email_field_label', $form_state->getValue('email_field_label'))
      ->set('contact_list_field_label', $form_state->getValue('contact_list_field_label'))
      ->set('agreement_checkbox_label', $form_state->getValue('agreement_checkbox_label'))
      ->set('submit_button_label', $form_state->getValue('submit_button_label'))
      ->set('successful_message', $form_state->getValue('successful_message'))
      ->set('failed_message', $form_state->getValue('failed_message'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
